'********************************************
'********************************************
' for 24V32 (32kBit EEPROM) on DS3231
' MMBasic 4.5 for Maximite/Duinomite
'
' eeWrite$: writes a string to address
' eeRead$: some bytes from address
' eeGet$: reads a string from address
' eeWriteN: writes a number (1 byte) to address
' eeReadN: reads a number (1 byte) from address
'   address may be between 0 to 4095
'
' eeSaveF: writes a 32 bit floatinpoint number
' eeLoadF: reads a 32 bit floatinpoint number
'   to/back from EEPROM using PEEK and POKE
'   uses eeWriteN respectively eeReadN
'
' eeErase: deletes the whole eeprom
' eeDump: shows content of eeprom
'
'
' EEPROM-Collection v0.5
' by twofingers 08-2014
' with parts from MMBasic Library
' (should be public domain)
'
'********************************************
'********************************************


Dim Z(20)
  'debug = 1 ' debugmode switch

  test = Pi
  Print "Save:       "test
  eeSaveF (32, test)
  test=0
  Print "Set to zero:"test
  eeLoadF (32, test)
  Print "Restored:   "test

  ' test an array
  For i = 1 To 20
     Z(i)=1/i
     Print "Save:       ",i, Z(i)
     eeSaveF (i*4, z(i))
     z(i)=0
     Print "Set to zero:",i, Z(i)
     eeLoadF (i*4, z(i))
     Print "Restored:   ", i, Z(i)
     Print
     wait
  Next i
  eeDump
End


' some string examples:
  T$="The BackShed eeprom test "

  'eeErase

  eeWrite$ 70, t$
  'eedump

  Print eeRead$(70,Len(t$))+"<"
  Print eeGet$(70)+"<"

  'eeWriteN 364, 70
  'Print eeReadN(364)

End


'******************************
Sub eeSaveF (address, v)
' Saves a floatingp. var to AT24C32
'******************************
  local i

  For i = 0 To 3
    eewriteN address+i, Peek(VAR v,i)
  Next i

End Sub



'******************************
Sub eeLoadF (address, v)
' Load a floatingp. var from AT24C32
'******************************
  local i

  For i = 0 To 3
    Poke VAR v,i, eeReadN(address+i)
  Next i

End Sub



'******************************
Sub eeWriteN (address, Number)
' Write a number to AT24C32
'******************************
  local MSB, LSB
  I2CADDR=&H57 ' 24C32-DS3231

  I2C open 100,100 'ENABLE I2C
  msb=Int(address/256) 'MSB
  lsb=address Mod 256  'LSB

  I2C write I2CADDR, 0, 3, msb, lsb, Number
  Pause 5
  If MM.I2C <>0 Then Print "Error! 1=NAK 2=TIMEOUT"; MM.I2C:End:EndIf
  I2C close

  If debug Then Print Number;" written into EEPROM."
End Sub


'******************************
Function eeReadN (address)
' Read a number from AT24C32
'******************************
  local MSB, LSB
  I2CADDR=&H57 ' 24C32-DS3231

  I2C open 100,100 'ENABLE I2C
  msb=Int(address/256) 'MSB
  lsb=address Mod 256  'LSB

  I2C write I2CADDR, 0,2,msb,lsb
  I2C read I2CADDR, 0,1,eeReadN

  If MM.I2C <>0 Then Print "ERROR! 1=NAK 2=TIMEOUT"; MM.I2C:eeReadN=-1:End:EndIf
  I2C close

End Function


'******************************
Sub eeWrite$ (address, text$)
' Writes a string to AT24C32
'******************************
  local a, x, msb, msb
  I2CADDR=&H57 ' 24C32-DS3231
  a=0

  If Len(text$)+address>4095 Then Print "Error: Address to high!":End

  I2C open 100,100 'ENABLE I2C
  For x= address To address+Len(text$) 'writes 0-byte at the end!
    a=a+1
    msb=Int(x/256) 'MSB
    lsb=x Mod 256  'LSB

    I2C write I2CADDR, 0, 3, msb, lsb, Asc(Mid$(text$,a,1))
    Pause 5
    If MM.I2C <>0 Then Print "Error! 1=NAK 2=TIMEOUT"; MM.I2C: End Sub    
  Next x
  I2C close

  If debug Then Print "At"address" string "+Chr$(34)+text$+Chr$(34)+"is written into EEPROM."
End Sub


'******************************
Function eeRead$ (address, bytes)
' Reads a string from AT24C32
'******************************
  local x, msb, lsb
  I2CADDR=&H57 ' 24C32-DS3231
  eeRead$=""

  If address>4095 Then Print "Error: Address to high!":End
  If bytes>255 Then Print "Error: String too long!":End

  I2C open 100,100 'ENABLE I2C
  For x= address To address+bytes-1

    msb=Int(x/256) 'MSB
    lsb=x Mod 256  'LSB

    I2C write I2CADDR, 0,2,msb,lsb
    I2C read I2CADDR, 0,1,tmp

    If MM.I2C <>0 Then Print "Error! 1=NAK 2=TIMEOUT"; MM.I2C: End Sub
    eeRead$=eeRead$+Chr$(tmp)

  Next x
  I2C close
End Function


'******************************
Function eeGet$ (address)
' Gets a zero-terminated string from AT24C32
'******************************
  local msb, lsb
  I2CADDR=&H57 ' 24C32-DS3231
  eeGet$=""

  If address>4095 Then Print "Error: Address to high!":End
  If bytes>255 Then Print "Error: String too long!":End

  I2C open 100,100 'ENABLE I2C

  Do

    msb=Int(address/256) 'MSB
    lsb=address Mod 256  'LSB

    I2C write I2CADDR, 0,2,msb,lsb
    I2C read I2CADDR, 0,1,tmp

    If MM.I2C <>0 Then Print "Error! 1=NAK 2=TIMEOUT"; MM.I2C: End Sub
    If tmp=0 Then Exit 'leave loop
    eeGet$=eeGet$+Chr$(tmp)
    address=address+1

  Loop While Len(eeGet$)<255
  I2C close
End Function


'**************************************
' ERASE WHOLE EEPROM (fill with zero)
' using PAGE WRITE
'**************************************
Sub eeErase
  local i, j, k, fill(34) ' fill will contain 2 bytes for address + 32 fillbytes
  I2CADDR=&H57 ' 24C32-DS3231
  K=&H00 'zero value or some fill value

  For i = 2 To 34:fill(i)=K: Next i

  Print "Erasing ..."
  I2C open 100,100 'ENABLE I2C
  For J=&h0 To &hF    'MSB
    For I=&h0 To &hFF Step 32'LSB
      fill(0)=j:fill(1)=i
      'fill(0)+(1) = MSB + LSB
      I2C write I2CADDR, 0,34,fill(0) 

      Pause 5 '? Write needs some time ...
      If MM.I2C <>0 Then Print "1=NAK 2=TIMEOUT"; MM.I2C,j,i:End:EndIf
    Next i
    L=(j*256+I)
    Print "Byte ";L, " ";Chr$(13);
  Next j
  I2C close
  Print "EEPROM HAS BEEN ERASED "

End Sub


'**************************************
' EEPROM komplett LESEN
' Shows content from AT24C32
'**************************************
Sub eedump

  Local i, j, y, L, RTCBUFF(4096)
  I2CADDR=&H57 ' 24C32-DS3231

  For I = 0 To 4095
     RTCBUFF(I)=&H0
  Next I

  Print " Reading buffer in ..."

  I2C open 100,100 'ENABLE I2C
    For J=&H0 To &HF
      For I=&h0 To &hFF
        L=(j*256+I)

        I2C write I2CADDR, 0,2,J,I
        I2C read I2CADDR, 0,1,RTCBUFF(L)

        If MM.I2C <>0 Then Print "ERROR! 1=NAK 2=TIMEOUT"; MM.I2C,j,i:End:EndIf
    Next I
  Next J

  I2C close

  Print " Show buffer (Press any key!) ..."

  For I=0 To 4095 Step 16
    Print "Byte ";Format$(I,"%4g");" ||";
    For y = 0 To 15
      Print Right$("0"+Hex$(RTCBUFF(I+y)),2);"|";
    Next y
    Print "|";
    For y = 0 To 15
      If RTCBUFF(I+y) >=32 And RTCBUFF(I+y)<=126 Then ' show ascii only
        C$=Chr$(RTCBUFF(I+y))
      Else
        C$=" "
      EndIf
      Print C$;
    Next y
    Print "|"
    wait
  Next I

End Sub


' wait for any key
Sub wait
  Do While Inkey$="":Loop
End Sub
